#
#   5B - Central game state
#

import os
from tempfile import mkstemp
from glob import glob
from level import Level
from reader import Reader
import settings

class Game(object):
	#  current_level          Level
	
	def __init__(self):
		#  ALL instance variables must be initialised here and not as
		#  class variables, so that game load/save works correctly.
		self.current_level = None
		self.current_level_dir = None

	def start_enabled(self):
		return self.level_present() and not self.level_running()
		
	def resume_enabled(self):
		return self.level_running()

	def abort_enabled(self):
		return self.level_running()

	def level_present(self):
		return self.current_level is not None
	
	def level_running(self):
		level = self.current_level
		if level:
			return level.playing()
		else:
			return False
	
	def start_level(self, testing = False):
		level = self.current_level
		if level:
			level.testing = testing
			level.restart()
	
	def abort_level(self):
		level = self.current_level
		if level:
			level.reset()

	def create_level(self):
		self.current_level = Level()

	def load_level(self, path):
		f = open(path, "rU")
		try:
			r = Reader(f, path)
			level = Level.create_from_file(r)
		finally:
			f.close()
		level.is_tutorial = level.get_name().startswith("+")
		self.current_level = level

	def save_level(self, path):
		level = self.current_level
		fd, temp_path = mkstemp(dir = os.path.dirname(path), text = True)
		f = os.fdopen(fd, "w")
		#f = open(path, "w")
		try:
			try:
				level.write_to_file(f)
			finally:
				f.close()
		except:
			os.unlink(temp_path)
			raise
		else:
			os.rename(temp_path, path)
		level.changed = False

	def get_current_playing_level_list(self, tutorial = True):
		level_dir = self.get_current_playing_level_dir()
		pat = os.path.join(level_dir, "*%s" % Level.level_file_suffix)
		#print "Game: globbing %r" % pat ###
		paths = glob(pat)
		if not tutorial:
			paths = [p for p in paths if not os.path.basename(p).startswith("+")]
		return paths

	def get_current_playing_level_dir(self):
		return self.current_level_dir \
			or settings.get_default_playing_level_dir()

	def get_current_custom_level_dir(self):
		return self.current_level_dir \
			or settings.get_default_editor_level_dir()

	def get_adjacent_level_path(self, way):
		paths = self.get_current_playing_level_list()
		try:
			i = paths.index(self.current_level.pathname)
		except ValueError:
			return None
		i += way
		if 0 <= i < len(paths):
			return paths[i]
